#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

"""

==========
aeresrh.py
==========

DESCRIPTION
===========

extract information from aeresrh.xls

.. option:: --ifile <file>

      file to read

.. only:: man

    Figure is visible on PDF and HTML documents only.

    .. only:: html or latex

       .. graphviz::

          digraph aeresrh {
                 rh [shape=diamond,
                     fontname=Courier,
                     label='{ifile}'

                 aeresrh [shape=box,
                     fontname=Courier,
                     color=blue,
                     URL="http://forge.ipsl.jussieu.fr/pulsation/browser/trunk/aeres/scripts/aeresrh.py",
                     label="${PROJECT}/aeres/scripts/aeresrh.py"];
             }

SEE ALSO
========

.. index:: ! Excel

https://secure.simplistix.co.uk/svn/xlrd/trunk/xlrd/doc/xlrd.html
http://www.simplistix.co.uk/presentations/python-excel.pdf


EXAMPLES
========

::

 aeresrh.py --ifile ${PROJECT}/data/aeresrh.xls

TODO
====

pb de type de fichier  ~/Downloads/personnel\ franc\314\247oise.xlsx

ne marche pas avec des chemins relatifs

append sur ifiles car liste permanent et liste thesard

comment j'ai la liste des gens partis au 30 juin 2012 avec publication (retraite, decs, mutation)

test with real file (encoding, francais, etc)

missing values

coding rules (pylint)

EVOLUTIONS
==========

$Id$

$URL$

- fplod 20120409

  * no more hard coded column number

- fplod 20120406

  * tst avec copie de vrai fichier ( ~/Downloads/personnel\ franc\314\247oise.xlsx)
    dans toto.xsl
- fplod 20120405
 
  * gestion des temps (arrive, dpart) (to be cont.)
  * creation draft
    thanks to http://www.portailsig.org/content/python-lire-et-ecrire-des-fichiers-microsoft-excel-application-quantum-gis
    and http://scienceoss.com/read-excel-files-from-python/
"""

import sys
import os
from os.path import dirname, basename
import glob
from optparse import OptionParser

import datetime

try:
    import xlrd
except ImportError:
    print("Failed to import xlrd from any known place")
    sys.exit(1)

from build_surname_id import build_surname_id
from build_firstname_id import build_firstname_id

def get_option_parser ():
    """parse CLI arguments

           :returns: parser
          :rtype: :class:`optparse.OptionParser`
    """

    parser = OptionParser('%prog [--verbose ] [--ifile file')
    parser.add_option ('-V', '--verbose',
        help='produce verbose output',
        action='store_true',
        default=False,
        dest='is_verbose')
    parser.add_option ('-i', '--ifile',
        help='file to be read',
	type='string',
        dest='ifile',
	default=None,
        action='store')

    return parser

def find_in_dict(d, criterion, lookup):
    """
    >>> people = {'pk1':{'firstname':'Brian', 'age':42}, 'pk2':{'firstname':'Alex', 'age':50}}
    >>> find_in_dict(people, 'age', 42)

    http://stackoverflow.com/questions/8987631/searching-2-dimensional-dictionary
    """

    result = []
    for key, val in d.iteritems():
        if criterion in val and val[criterion] == lookup:
            result.append((key,val))
    return result

def readrh(ifile, is_verbose):

    """
    read ifile

    return surname, firstname, arrival,  publi
    """

    if is_verbose == True:
        print('iii : read %s' % ifile)

    # ouverture du fichier Excel
    wb = xlrd.open_workbook(ifile, formatting_info=True)

    # feuilles dans le classeur
    if is_verbose == True:
        print('sheets name %s' % ( wb.sheet_names()))

    # lecture des donnes dans la premire feuille
    sh = wb.sheet_by_index(0)

    #
    if is_verbose == True:
        for rownum in range(sh.nrows):
            print('values of in row %d : %s' % (rownum, sh.row_values(rownum)))

    if is_verbose == True:
        for colnum in range(sh.ncols):
            print('values of in col %d : %s' % (colnum, sh.col_values(colnum)))

    # detect title row
    for rownum in range(sh.nrows):
        if sh.row_values(rownum)[0] == 't':
	    titlerow = rownum 
    if is_verbose == True:
	    print('titlerow : %d ' % (titlerow))

    # deduce first usefull row
    firstrow = titlerow + 1
    #
    # detect last usefull row
    uselessrow = []
    for rownum in range(titlerow,sh.nrows):
        if sh.row_values(rownum)[0] == '': 
	    uselessrow.append(rownum)
    lastrow = uselessrow[0]
    if is_verbose == True:
	    print('lastrow : %d ' % (lastrow))

    # detect surname column
    for colnum in range(sh.ncols):
        if sh.col_values(colnum)[titlerow] == 'nom':
	    colsurname = colnum
    if is_verbose == True:
	    print('colsurname: %d ' % (colsurname))

    # detect firstname column
    for colnum in range(sh.ncols):
	if sh.col_values(colnum)[titlerow] == 'prenom':
	    colfirstname = colnum
    if is_verbose == True:
	    print('colfirstname: %d ' % (colfirstname))

    # detect arrival column
    for colnum in range(sh.ncols):
	if sh.col_values(colnum)[titlerow] == u'arrive':
	    colarrival = colnum
    if is_verbose == True:
	    print('colarrival: %d ' % (colarrival))

    # detect publi column
    for colnum in range(sh.ncols):
	if sh.col_values(colnum)[titlerow] == u'publi':
	    colpubli = colnum
    if is_verbose == True:
	    print('colpubli: %d ' % (colpubli))

    datemode = 0 

    surname = sh.col_values(colsurname)[firstrow:lastrow]
    print ('surname 0000 %s ' % surname)

    if is_verbose == True:
       print(' nb surname %d' % len(surname))
       print(' surname %s' % surname)

    firstname = sh.col_values(colfirstname)[firstrow:lastrow]

    if is_verbose == True:
       print(' nb firstname %d' % len(firstname))
       print(' firstname %s' % firstname)

    temp = sh.col_values(colarrival)[firstrow:lastrow]
    arrival = []
    for rownum in range(len(temp)):
        if is_verbose == True:
            print(' temp arrival AAA%sAAA' % temp[rownum])
        #++ pb sh.datemodearrival = datetime.datetime(*xlrd.xldate_as_tuple(temp[rownum],sh.datemode))
        if temp[rownum] == '':
            print('eee : pas de date arrivee colonne %d ' % rownum)
        else:
            print('iii : date arrivee pour %s ' % surname[rownum])
            #arrival.append(datetime.datetime(*xlrd.xldate_as_tuple(temp[rownum],datemode)))

    if is_verbose == True:
       print(' nb arrival %d' % len(arrival))
       print(' arrival %s' % arrival)

    publi = sh.col_values(colpubli)[firstrow:lastrow]

    if is_verbose == True:
       print(' nb publi %d' % len(publi))
       print(' publi %s' % publi)

    return surname, firstname, arrival, publi

def aeresrh():
    """main
    """
    try:
        parser = get_option_parser ()
        (fromcli, args) = parser.parse_args()
    except IOError, msg:
        parser.error(str(msg))

    is_verbose = fromcli.is_verbose
    ifile = fromcli.ifile
    if fromcli.is_verbose == True:
       print ('ifile : %s' % (ifile))

    surname, firstname, arrival, publi = readrh(ifile, is_verbose)
    #
    # build surname id
    surname_id = build_surname_id(surname)
    firstname_id = build_firstname_id(firstname)
    #
    # populate dictionary
    #++people = {}
    #++for index, onesurname in enumarate(surname):
    #++   print (' surname %s' % (surname))
    #++   people[n] = surname[index]

    find_in_dict(people, 'surname', 'Pinsard')

# Run main, if called from the command line
if __name__ == '__main__':
    aeresrh()

